# -*- coding: utf-8 -*-
"""Storage components."""

# zope imports
from persistent.dict import PersistentDict
from zope.interface import implementer

# local imports
from ps.zope.i18nfield import (
    interfaces,
    utils,
)

_marker = dict()
KEY_DEFAULT = u'__default_value'


@implementer(interfaces.II18NDict)
class I18NDict(PersistentDict):
    """A custom dictionnary handling a default value."""

    @classmethod
    def from_text(cls, value):
        """Create a new I18nDict from a str or unicode value."""
        klass = cls()
        klass[KEY_DEFAULT] = value
        return klass

    def __init__(self, *args, **kw):
        super(I18NDict, self).__init__()
        self.update(*args, **kw)
        self.default_language = None
        self.required = False

    def __unicode__(self):
        """Return the unicode representation of the dictionary by first trying
        to access the value for the current selected language (i.e. from the
        request). If no value exists for this language and the associated
        schema field is required, try to find the best default fallback value
        available.
        """
        lang_req = utils.get_language()
        return unicode(self.get_for_language(lang_req))

    def __str__(self):
        """Return the utf-8 encoded respresentation of the dictionary."""
        return unicode(self).encode('utf-8')

    def __nonzero__(self):
        """Return whether the dictionary is considered empty or not."""
        lang_req = utils.get_language()
        return self.get_for_language(lang_req) is not None

    def __setitem__(self, key, value):
        """Set a new value, but first doing a validity check."""
        if not value:
            return
        if key != KEY_DEFAULT and key not in utils.available_languages():
            return
        super(I18NDict, self).__setitem__(key, value)

    def get_default_value(self):
        """Returns the best available default value based on the following
        order:
        1. self.default_language
        2. Context/application default (utility)
        3. First non-empty value
        4. None
        """
        result = self.get(self.default_language)
        if not result:
            result = self.get(utils.get_default_language())
            if not result:
                for value in sorted(self.values()):
                    if value:
                        return value

        return result

    def get_for_language(self, language):
        """Get the value for a specific language using the appropriate default
        if it is a required field.
        """
        result = self.get(language, self.get(KEY_DEFAULT))
        if not result and self.required:
            result = self.get_default_value()
        return result

    def add(self, language, value):
        """Add a value for the given language."""
        self[language] = value

    def remove(self, language):
        """Remove the value for the given language."""
        if language in self:
            del self[language]

    def update(self, *args, **kwargs):
        """Update the current dict with either a list of tuples or another
        dictionary."""
        if args:
            if len(args) > 1:
                raise TypeError('update expected at most 1 arguments, '
                                'got %d' % len(args))
            other = dict(args[0])
            for key in other:
                self[key] = other[key]
        for key in kwargs:
            self[key] = kwargs[key]

    def copy(self):
        """Return an exact copy of the given dict with all associated
        properties."""
        result = I18NDict(**self)
        result.default_language = self.default_language
        result.required = self.required
        return result

    def to_dict(self):
        """Return a plain dict representation of the object."""
        return dict(**self)

    def to_text(self):
        """Return a string of all text values appended together."""
        return u' '.join(self.values())



# -*- coding: utf-8 -*-

# zope imports
from zope import schema

# local imports
from ps.zope.i18nfield import (
    interfaces,
    storage,
    utils,
)

_marker = dict()


class I18NFieldProperty(schema.fieldproperty.FieldProperty):
    """Base class for I18N properties."""

    def __init__(self, field, name=None, value_converters=None):
        if not interfaces.II18NField.providedBy(field):
            raise ValueError(
                u'Provided field must implement II18NField interface.'
            )
        if name is None:
            name = field.__name__
        self.__field = field
        self.__name = name
        if value_converters is None:
            self.__value_converters = ()
        else:
            self.__value_converters = value_converters

    def __get__(self, instance, klass):
        if instance is None:
            return self
        value = instance.__dict__.get(self.__name, _marker)
        if value is _marker:
            field = self.__field.bind(instance)
            value = getattr(field, 'default', _marker)
            if value is _marker:
                raise AttributeError(self.__name)
        return value

    def __set__(self, instance, value):
        # prepare the values
        value = self.prepare_dict(value)
        old_value = getattr(instance, self.__name, None)
        if old_value is None and isinstance(value, basestring):
            value = storage.I18NDict.from_text(value)
        if isinstance(value, storage.I18NDict):
            validate_dict = value.to_dict()
            storage_dict = value
            storage_dict.clear()
        else:
            validate_dict = value
            storage_dict = storage.I18NDict()

        assert isinstance(validate_dict, dict), 'validate_dict must be dict'
        assert isinstance(storage_dict, storage.I18NDict), \
            'storage_dict must be I18NDict'

        # field validation
        field = self.__field.bind(instance)
        field.validate(validate_dict)
        if field.readonly and self.__name in instance.__dict__:
            raise ValueError(self.__name, u'Field is readonly')

        # set the new I18nDict
        storage_dict.default_language = field._default_language
        storage_dict.required = field.required
        storage_dict.update(validate_dict)
        instance.__dict__[self.__name] = storage_dict

    def __getattr__(self, name):
        return getattr(self.__field, name)

    def prepare_dict(self, value):
        """Prepare the values of the dict by removing any empty values and
        passing the remaining values through any converters associated with
        this field.

        :param value: Value that will be set with the FieldProperty
        :type value: dict
        :returns: The prepared values
        :rtype: dict
        """
        if value is None:
            return {}
        if isinstance(value, dict) or isinstance(value, storage.I18NDict):
            for lang in value.keys():
                if not value[lang]:
                    del value[lang]
            for converter in self.__value_converters:
                for lang in value:
                    value[lang] = converter(value[lang])
        return value


class I18NTextProperty(I18NFieldProperty):
    """I18n property to handle Text and TextLine values"""

    def __init__(self, field, name=None):
        super(I18NTextProperty, self).__init__(
            field,
            name,
            value_converters=(utils.to_unicode,)
        )



# -*- coding: utf-8 -*-
"""I18N widget for z3c.form."""

# python imports
from copy import copy
import json

# zope imports
from persistent.dict import PersistentDict
from plone.memoize.view import memoize
from z3c.form.browser import (
    text,
    textarea,
)
from z3c.form.browser.widget import HTMLFormElement
from z3c.form.interfaces import (
    NO_VALUE,
    IDataConverter,
    IFieldWidget,
    IFormLayer,
)
from z3c.form.widget import (
    FieldWidget,
    Widget,
)
from zope.component import (
    adapter,
    queryUtility,
)
from zope.i18n import translate
from zope.interface import (
    implementer,
    implementsOnly,
)
from zope.publisher.browser import BrowserView
from zope.publisher.interfaces import (
    IPublishTraverse,
    NotFound,
)
from zope.security.proxy import removeSecurityProxy

# local imports
from ps.zope.i18nfield import (
    interfaces,
    storage,
    utils,
)
from ps.zope.i18nfield.i18n import _
from ps.zope.i18nfield.z3cform.interfaces import (
    II18NTextAreaWidget,
    II18NTextWidget,
    II18NWidget,
)


class I18NWidgetProperty(object):
    """Base class for I18N widgets properties."""

    def __init__(self, name):
        self.__name = name

    def __get__(self, instance, klass):
        return instance.__dict__.get(self.__name, None)

    def __set__(self, instance, value):
        instance.__dict__[self.__name] = value
        for widget in instance.widgets.values():
            setattr(widget, self.__name, value)


class I18NWidget(HTMLFormElement, Widget):
    """Base class for all I18N widgets."""
    implementsOnly(II18NWidget)

    default_widget = None
    show_label = True
    default_label = _(u'Default')
    default_info = _(
        u'Please copy the default text to the corresponding language.'
    )
    option_select_language = _(u'Select language')
    button_add_language = _(u'Add translation')

    # IHTMLCoreAttributes properties
    klass = 'i18n-widget'
    style = I18NWidgetProperty('style')
    # IHTMLEventsAttributes properties
    onclick = I18NWidgetProperty('onclick')
    ondblclick = I18NWidgetProperty('ondblclick')
    onmousedown = I18NWidgetProperty('onmousedown')
    onmouseup = I18NWidgetProperty('onmouseup')
    onmouseover = I18NWidgetProperty('onmouseover')
    onmousemove = I18NWidgetProperty('onmousemove')
    onmouseout = I18NWidgetProperty('onmouseout')
    onkeypress = I18NWidgetProperty('onkeypress')
    onkeydown = I18NWidgetProperty('onkeydown')
    onkeyup = I18NWidgetProperty('onkeyup')
    # IHTMLFormElement properties
    disabled = I18NWidgetProperty('disabled')
    tabindex = I18NWidgetProperty('tabindex')
    onfocus = I18NWidgetProperty('onfocus')
    onblur = I18NWidgetProperty('onblur')
    onchange = I18NWidgetProperty('onchange')

    def __init__(self, request):
        super(I18NWidget, self).__init__(request)

    def available_languages(self):
        if self.value and self.value.keys():
            langs = self.value.keys()
            if storage.KEY_DEFAULT in langs and len(langs) == 1:
                langs.append(utils.get_language(request=self.request))
            return langs
        return [self.current()]

    def addable_languages(self):
        languages = self.languages()
        sorted_langs = self.sorted_languages()
        available_langs = self.available_languages()
        result = [
            {'lang': str(l), 'name': languages.get(l)}
            for l in sorted_langs
            if l not in available_langs
        ]
        return json.dumps(result)

    @memoize
    def sorted_languages(self):
        available = utils.available_languages()
        tmp_languages = sorted([unicode(key) for key in available])
        languages = []
        if u'en' in tmp_languages:
            tmp_languages.remove(u'en')
            languages.append(u'en')
        if u'es' in tmp_languages:
            tmp_languages.remove(u'es')
            languages.append(u'es')
        languages.extend(tmp_languages)
        return languages

    @memoize
    def languages(self):
        utility = queryUtility(interfaces.ILanguageAvailability)
        if utility is None:
            return None
        return dict(utility.getLanguageListing())

    def current(self):
        lang = utils.get_language(request=self.request)
        if self.value is not None:
            has_default = storage.KEY_DEFAULT in self.value
            return has_default and storage.KEY_DEFAULT or lang
        return lang

    def default_value(self):
        if self.value is not None:
            return self.value.get(storage.KEY_DEFAULT, None)

    def update(self):
        super(I18NWidget, self).update()
        widgets = self.widgets = {}
        langs = self.sorted_languages()
        for lang in langs:
            widget = widgets[lang] = self.default_widget(self.request)
            self.initWidget(widget, lang)
        for lang in langs:
            widget = self.widgets[lang]
            self.updateWidget(widget, lang)
            widget.update()
        default_value = self.default_value()
        if default_value is not None:
            lang = storage.KEY_DEFAULT
            widget = widgets[lang] = self.default_widget(self.request)
            self.initWidget(widget, lang)
            widget = self.widgets[lang]
            self.updateWidget(widget, lang)
            widget.update()

    def initWidget(self, widget, language):
        widget.id = str('{0}.{1}'.format(self.name, language))
        widget.form = self.form
        widget.mode = self.mode
        widget.ignoreContext = self.ignoreContext
        widget.ignoreRequest = self.ignoreRequest
        widget.field = self.field.value_type
        widget.name = str('{0}.i18n.{1}'.format(self.name, language))
        widget.label = self.label
        widget.lang = language

    def extract(self, default=NO_VALUE):
        """See z3c.form.interfaces.IWidget."""
        form_keys = self.request.form.keys()
        can_add = '{0}.button_add'.format(self.name) in form_keys
        available_languages = copy(self.sorted_languages())
        available_languages.append(storage.KEY_DEFAULT)
        result = {}

        for key in form_keys:
            if not key.startswith(self.name):
                continue
            lang = key.split('.').pop()
            if lang == 'add' and can_add:
                lang = self.request.get(key, default)
                if isinstance(lang, list):
                    lang = lang.pop()
                if lang in available_languages:
                    result[lang] = u''
            else:
                if lang in available_languages:
                    result[lang] = self.request.get(key, default)
        if len(result.keys()) < 1:
            result = default
        return result

    def updateWidget(self, widget, language):
        widget.value = self.getValue(language)

    def getWidget(self, language):
        return self.widgets.get(language)

    def getValue(self, language):
        self.value = removeSecurityProxy(self.value)
        if not isinstance(self.value, dict) and \
           not isinstance(self.value, PersistentDict):
            converter = IDataConverter(self)
            try:
                self.value = converter.toFieldValue(self.value)
            except:
                self.value = {}
        if self.value is not None:
            return self.value.get(language)

    def hasValue(self, language):
        return bool(self.getValue(language))

    def render(self):
        return super(I18NWidget, self).render()

    @property
    def button_add_language_i18n(self):
        return translate(self.button_add_language, context=self.request)


class I18NTextWidget(I18NWidget, text.TextWidget):
    """I18N text input type implementation."""
    implementsOnly(II18NTextWidget)

    default_widget = text.TextWidget

    maxlength = I18NWidgetProperty('maxlength')
    size = I18NWidgetProperty('size')

    def updateWidget(self, widget, language):
        super(I18NTextWidget, self).updateWidget(widget, language)
        widget.maxlength = widget.field.max_length


@adapter(interfaces.II18NTextLineField, IFormLayer)
@implementer(IFieldWidget)
def I18NTextFieldWidget(field, request):
    """IFieldWidget factory for I18NTextWidget."""
    return FieldWidget(field, I18NTextWidget(request))


class I18NTextAreaWidget(I18NWidget, textarea.TextAreaWidget):
    """I18N text input type implementation."""
    implementsOnly(II18NTextAreaWidget)

    default_widget = textarea.TextAreaWidget

    rows = I18NWidgetProperty('rows')
    cols = I18NWidgetProperty('cols')
    readonly = I18NWidgetProperty('readonly')
    onselect = I18NWidgetProperty('onselect')


@adapter(interfaces.II18NTextField, IFormLayer)
@implementer(IFieldWidget)
def I18NTextAreaFieldWidget(field, request):
    """IFieldWidget factory for I18NTextWidget."""
    return FieldWidget(field, I18NTextAreaWidget(request))


@implementer(IPublishTraverse)
class WidgetAjax(BrowserView):

    def __init__(self, context, request):
        context = removeSecurityProxy(context)
        super(WidgetAjax, self).__init__(context, request)
        self.language = None

    def publishTraverse(self, request, name):
        if self.language is None:
            self.language = name
        else:
            raise NotFound(self, name, request)
        return self

    def __call__(self):
        if self.language is None:
            return
        widget = self.context.getWidget(self.language)
        if widget is not None:
            return widget.render()



# -*- coding: utf-8 -*-
"""Integration tests for the I18NFieldIndex."""

# python imports
try:
    import unittest2 as unittest
except ImportError:
    import unittest

# zope imports
from zope.component import provideUtility
from zope.interface import implementer

# local imports
from ps.zope.i18nfield import index, interfaces, storage, utils


@implementer(interfaces.ILanguageAvailability)
class LanguageAvailability(object):

    def getDefaultLanguage(self, combined=False):
        return u'en'

    def getAvailableLanguages(self, combined=False):
        return [u'de', u'en', u'es', u'fr', u'pt']

    def getLanguages(self, combined=False):
        pass

    def getLanguageListing(self, combined=False):
        pass


class TestI18NFieldIndex(unittest.TestCase):
    """Test case for the I18NFieldIndex."""

    def setUp(self):
        super(TestI18NFieldIndex, self).setUp()
        provideUtility(LanguageAvailability())
        self._old_func = utils.get_language

    def tearDown(self):
        super(TestI18NFieldIndex, self).tearDown()
        utils.get_language = self._old_func

    def test_do_index(self):
        """Test that the I18NDict values are indexed properly."""
        idx = index.I18NFieldIndex()
        data = storage.I18NDict({u'en': u'English', u'de': u'Deutsch'})
        idx.doIndex(111, data)
        self.assertEqual(len(idx._indices), 2)

        # English
        sub_idx = idx._indices[u'en']
        self.assertEqual(sub_idx.documentCount(), 1)
        self.assertEqual(sub_idx.wordCount(), 1)

        data2 = storage.I18NDict({
            u'en': u'English with more words',
            u'es': u'Español con más palabras',
            u'de': u'Deutsch'
        })
        idx.doIndex(222, data2)
        self.assertEqual(len(idx._indices), 3)

        # English
        sub_idx = idx._indices[u'en']
        self.assertEqual(sub_idx.documentCount(), 2)
        self.assertEqual(sub_idx.wordCount(), 2)
        self.assertIn(u'English', list(sub_idx._rev_index.values()))
        self.assertIn(
            u'English with more words',
            list(sub_idx._rev_index.values()),
        )
        # Spanish
        sub_idx = idx._indices[u'es']
        self.assertEqual(sub_idx.documentCount(), 1)
        self.assertEqual(sub_idx.wordCount(), 1)
        self.assertIn(
            u'Español con más palabras',
            list(sub_idx._rev_index.values()),
        )
        # German
        sub_idx = idx._indices[u'de']
        self.assertEqual(sub_idx.documentCount(), 2)
        self.assertEqual(sub_idx.wordCount(), 1)
        self.assertIn(u'Deutsch', list(sub_idx._rev_index.values()))

    def test_do_index_required(self):
        """Test that the I18NDict values are indexed properly."""
        idx = index.I18NFieldIndex()
        data = storage.I18NDict({u'en': u'English', u'de': u'Deutsch'})
        data.required = True
        idx.doIndex(111, data)
        self.assertEqual(len(idx._indices), 5)

        # English
        sub_idx = idx._indices[u'en']
        self.assertEqual(sub_idx.documentCount(), 1)
        self.assertEqual(sub_idx.wordCount(), 1)
        self.assertIn(u'English', list(sub_idx._rev_index.values()))
        # Portuguese
        sub_idx = idx._indices[u'pt']
        self.assertEqual(sub_idx.documentCount(), 1)
        self.assertEqual(sub_idx.wordCount(), 1)
        self.assertIn(u'English', list(sub_idx._rev_index.values()))

        data2 = storage.I18NDict({
            u'en': u'English with more words',
            u'es': u'Español con más palabras',
            u'de': u'Deutsch'
        })
        data2.required = True
        idx.doIndex(222, data2)
        self.assertEqual(len(idx._indices), 5)

        # English
        sub_idx = idx._indices[u'en']
        self.assertEqual(sub_idx.documentCount(), 2)
        self.assertEqual(sub_idx.wordCount(), 2)
        self.assertIn(u'English', list(sub_idx._rev_index.values()))
        self.assertIn(
            u'English with more words',
            list(sub_idx._rev_index.values()),
        )
        # Spanish
        sub_idx = idx._indices[u'es']
        self.assertEqual(sub_idx.documentCount(), 2)
        self.assertEqual(sub_idx.wordCount(), 2)
        self.assertIn(u'English', list(sub_idx._rev_index.values()))
        self.assertIn(
            u'Español con más palabras',
            list(sub_idx._rev_index.values()),
        )
        # German
        sub_idx = idx._indices[u'de']
        self.assertEqual(sub_idx.documentCount(), 2)
        self.assertEqual(sub_idx.wordCount(), 1)
        self.assertIn(u'Deutsch', list(sub_idx._rev_index.values()))
        # Portuguese
        sub_idx = idx._indices[u'pt']
        self.assertEqual(sub_idx.documentCount(), 2)
        self.assertEqual(sub_idx.wordCount(), 2)
        self.assertIn(u'English', list(sub_idx._rev_index.values()))
        self.assertIn(
            u'English with more words',
            list(sub_idx._rev_index.values()),
        )

    def test_do_unindex(self):
        """Test that the I18NDict values are indexed properly."""
        idx = index.I18NFieldIndex()
        data = storage.I18NDict({u'en': u'English', u'de': u'Deutsch'})
        idx.doIndex(111, data)
        self.assertEqual(len(idx._indices), 2)

        data2 = storage.I18NDict({
            u'en': u'English with more words',
            u'es': u'Español con más palabras',
            u'de': u'Deutschland'
        })
        idx.doIndex(222, data2)
        self.assertEqual(len(idx._indices), 3)

        idx.doUnIndex(222)
        self.assertEqual(len(idx._indices), 3)

        # English
        sub_idx = idx._indices[u'en']
        self.assertEqual(sub_idx.documentCount(), 1)
        self.assertEqual(sub_idx.wordCount(), 1)
        self.assertIn(u'English', list(sub_idx._rev_index.values()))
        # Spanish
        sub_idx = idx._indices[u'es']
        self.assertEqual(sub_idx.documentCount(), 0)
        self.assertEqual(sub_idx.wordCount(), 0)
        # English
        sub_idx = idx._indices[u'de']
        self.assertEqual(sub_idx.documentCount(), 1)
        self.assertEqual(sub_idx.wordCount(), 1)
        self.assertIn(u'Deutsch', list(sub_idx._rev_index.values()))

        # clear all indices
        idx.clear()
        self.assertEqual(len(idx._indices), 0)

    def test_sort(self):
        """Test the normal sort function."""
        idx = index.I18NFieldIndex()
        data = storage.I18NDict({
            u'en': u'Title in english',
            u'es': u'El título',
        })
        idx.doIndex(111, data)

        data2 = storage.I18NDict({
            u'en': u'English words',
            u'es': u'Título en español',
            u'de': u'Deutsch'
        })
        idx.doIndex(222, data2)

        # request language English
        utils.get_language = lambda: u'en'
        self.assertEqual(list(idx.sort([111, 222])), [222, 111])

        # request language Spanish
        utils.get_language = lambda: u'es'
        self.assertEqual(list(idx.sort([111, 222])), [111, 222])

        # request language German
        utils.get_language = lambda: u'de'
        self.assertEqual(list(idx.sort([111, 222])), [222])

    def test_sort_no_values(self):
        """Test the sort function when no values are indexed."""
        idx = index.I18NFieldIndex()
        self.assertEqual(list(idx.sort([])), [])



# -*- coding: utf-8 -*-
"""Integration tests for the I18NDict."""

# python imports
try:
    import unittest2 as unittest
except ImportError:
    import unittest

# zope imports
from zope.component import provideUtility
from zope.interface import implementer

# local imports
from ps.zope.i18nfield import (
    interfaces,
    storage,
    utils,
)


@implementer(interfaces.ILanguageAvailability)
class LanguageAvailability(object):

    def getDefaultLanguage(self, combined=False):
        return u'en'

    def getAvailableLanguages(self, combined=False):
        return [u'de', u'en', u'es', u'fr', u'pt']

    def getLanguages(self, combined=False):
        pass

    def getLanguageListing(self, combined=False):
        pass


class TestI18NDict(unittest.TestCase):
    """Test case for the I18n language dictionary."""

    def setUp(self):
        super(TestI18NDict, self).setUp()
        provideUtility(LanguageAvailability())
        self._old_func = utils.get_language

    def tearDown(self):
        super(TestI18NDict, self).tearDown()
        utils.get_language = self._old_func

    def test_standard(self):
        """Test that the dictionary access with no default language."""
        data = storage.I18NDict()
        self.assertIsNone(data.get(u'en'))
        data[u'en'] = u'English'
        self.assertEqual(data.get(u'en'), u'English')
        data[u'es'] = u'Español'
        self.assertEqual(data.get(u'es'), u'Español')
        self.assertEqual(len(data.keys()), 2)

    def test_default(self):
        """Test the default value method with a default language set."""
        data = storage.I18NDict()
        data.default_language = u'es'

        # no values - fallback is None
        self.assertIsNone(data.get_default_value())

        # one value - fallback is the first value found
        data[u'fr'] = u'French'
        self.assertEqual(data.get_default_value(), u'French')

        # two values - fallback comes from global default (ILanguageSettings)
        data[u'en'] = u'English'
        self.assertEqual(data.get_default_value(), u'English')

        # three values - fallback comes from I18nDict default language
        data[u'es'] = u'Español'
        self.assertEqual(data.get_default_value(), u'Español')

        # set to non-existent default language - default from ILanguageSettings
        data.default_language = u'de'
        self.assertEqual(data.get_default_value(), u'English')

        # default comes from global registry
        # self.assertEqual(data.get_default_value(), u'French')

        # default comes from I18nDict internally saved default language
        data.default_language = u'en'
        self.assertEqual(data.get_default_value(), u'English')

    def test_no_default(self):
        """Test the default value method with no default langauge set."""
        data = storage.I18NDict()

        # no values - default is None
        self.assertIsNone(data.get_default_value())

        # one value - fallback is first value found
        data[u'fr'] = u'French'
        self.assertEqual(data.get_default_value(), u'French')

        # two values - global registry default is used
        data[u'en'] = u'English'
        self.assertEqual(data.get_default_value(), u'English')

        # three values - global registry default is still used
        data[u'es'] = u'Español'
        self.assertEqual(data.get_default_value(), u'English')

    def test_unicode_representation(self):
        """Test the __unicode__ method of the dictionary."""
        data = storage.I18NDict()
        data.required = True

        # no request, no values
        self.assertEqual(unicode(data), u'None')

        # no request - fallback to first value
        data[u'fr'] = u'French'
        self.assertEqual(unicode(data), u'French')

        # no request - fallback to global default (English)
        data[u'en'] = u'English'
        data[u'es'] = u'Español'
        self.assertEqual(unicode(data), u'English')

        # request language Spanish
        utils.get_language = lambda: u'es'
        self.assertEqual(unicode(data), u'Español')

        # request language French
        utils.get_language = lambda: u'fr'
        self.assertEqual(unicode(data), u'French')

        # request language English
        utils.get_language = lambda: u'en'
        self.assertEqual(unicode(data), u'English')

    def test_string_representation(self):
        """Test the __str__ method of the dictionary."""
        data = storage.I18NDict()
        data.default_language = u'es'
        data.required = True

        # no request, no values
        self.assertEqual(str(data), 'None')

        # no request - fallback to first value
        data[u'fr'] = u'French'
        self.assertEqual(str(data), 'French')

        # no value for request language - fallback to internal I18nDict default
        data[u'en'] = u'English'
        data[u'es'] = u'Español'
        utils.get_language = lambda: u'pt'
        self.assertEqual(str(data), u'Español'.encode('utf-8'))

        # request language Spanish
        utils.get_language = lambda: u'es'
        self.assertEqual(str(data), u'Español'.encode('utf-8'))

        # request language French
        utils.get_language = lambda: u'fr'
        self.assertEqual(str(data), 'French')

        # request language English
        utils.get_language = lambda: u'en'
        self.assertEqual(str(data), 'English')

    def test_not_required(self):
        """Test the __unicode__ method without a required field."""
        data = storage.I18NDict()
        data.default_language = u'es'
        data.required = False
        data[u'fr'] = u'French'
        data[u'de'] = u'German'
        data[u'es'] = u'Español'

        # no request, no value for default language 'en'
        self.assertEqual(unicode(data), u'None')

        # with required
        data.required = True
        self.assertEqual(unicode(data), u'Español')

    def test_from_text(self):
        """Test the from_text method."""
        data = storage.I18NDict.from_text(u'Test value')
        self.assertEqual(data, {u'__default_value': u'Test value'})
        self.assertEqual(unicode(data), u'Test value')

    def test_to_dict(self):
        original = storage.I18NDict()
        original[u'en'] = u'English'
        original[u'es'] = u'Español'
        self.assertEqual(len(original), 2)
        copy = original.to_dict()
        self.assertEqual(len(copy), 2)
        self.assertIn(u'en', copy.keys())
        self.assertIn(u'Español', copy.values())

    def test_copy(self):
        original = storage.I18NDict()
        original.default_language = u'es'
        original.required = True
        original[u'en'] = u'English'
        original[u'es'] = u'Español'
        self.assertEqual(len(original), 2)
        copy = original.copy()
        self.assertEqual(len(copy), 2)
        self.assertIn(u'es', copy.keys())
        self.assertIn(u'English', copy.values())
        self.assertEqual(copy.default_language, original.default_language)
        self.assertEqual(copy.required, original.required)

    def test_set_item(self):
        """Test the checks from __setitem__ method."""
        data = storage.I18NDict()
        data[u'fr'] = u'French'
        self.assertIn(u'fr', data)
        data[u'__default_value'] = u'Default value'
        self.assertIn(u'Default value', data.values())
        data[u'non-existent'] = u'Invalid'
        self.assertEqual(len(data), 2)
        data[u'en'] = u''
        self.assertEqual(len(data), 2)
        self.assertNotIn(u'en', data)

    def test_update(self):
        """Test the update method."""
        data = storage.I18NDict({u'en': u'English', u'invalid': u'blah'})
        self.assertEqual(len(data), 1)
        self.assertIn(u'en', data)

        data.update({
            u'es': u'Español',
            u'fr': u'François',
            u'fake': u'Non-value',
        })
        self.assertEqual(len(data), 3)
        self.assertIn(u'Español', data.values())
        self.assertIn(u'François', data.values())

        data.update([
            (u'de', u'Deutsch'),
            (u'pt', u'Português'),
            (u'slang', u'Jibber jabber'),
        ])
        self.assertEqual(len(data), 5)
        self.assertIn(u'de', data.keys())
        self.assertIn(u'pt', data.keys())

    def test_add(self):
        """Test the add method."""
        data = storage.I18NDict()
        data.add(u'de', u'Deutsch')
        self.assertEqual(len(data), 1)
        self.assertIn(u'de', data)
        data.add(u'fake', u'Fake language')
        self.assertEqual(len(data), 1)

    def test_remove(self):
        """Test the remove method."""
        data = storage.I18NDict({u'en': u'English', u'de': u'Deutsch'})
        self.assertEqual(len(data), 2)
        data.remove(u'de')
        self.assertEqual(len(data), 1)
        self.assertNotIn(u'de', data)
        data.remove(u'not')
        self.assertEqual(len(data), 1)

    def test_to_text(self):
        """Test the to_text method."""
        data = storage.I18NDict({u'en': u'English', u'de': u'Deutsch'})
        self.assertIn(u'English', data.to_text())
        self.assertIn(u'Deutsch', data.to_text())
        data[u'es'] = u'El texto en español'
        self.assertIn(u'El texto en español', data.to_text())

    def test_nonzero_not_required(self):
        """Test the __nonzero__ method."""
        data = storage.I18NDict()
        data.required = False

        # no request, no values
        self.assertFalse(bool(data))

        # no request - no value for default language
        data[u'fr'] = u'Français'
        self.assertFalse(bool(data))

        # no request - no value for default language
        data[u'es'] = u'Español'
        self.assertFalse(bool(data))

        # no request - with a value for default language
        data[u'en'] = u'English'
        self.assertTrue(bool(data))

    def test_nonzero_required(self):
        """Test the __nonzero__ method."""
        data = storage.I18NDict()
        data.required = True

        # no request, no values
        self.assertFalse(bool(data))

        # no request - fallback to first value found
        data[u'fr'] = u'Français'
        self.assertTrue(bool(data))

        # no request - fallback to first value found
        data[u'es'] = u'Español'
        self.assertTrue(bool(data))

        # no request - with a value for default language
        data[u'en'] = u'English'
        self.assertTrue(bool(data))



# -*- coding: utf-8 -*-
"""A zope.schema field for inline translations."""

# zope imports
from zope.interface import implementer
from zope.schema import (
    Dict,
    Text,
    TextLine,
)
from zope.schema.interfaces import RequiredMissing

# local imports
from ps.zope.i18nfield import (
    interfaces,
    storage,
)


@implementer(interfaces.II18NField)
class I18NField(Dict):
    """Base class for I18n schema fields

    A I18n field is a mapping object for which keys are languages and values
    are effective values of the given attribute. Selected values can be
    returned selectively (for editing), or automatically (to be displayed
    in an HTML page) based on user's browser's language settings.
    """

    def __init__(self, default_language=None, key_type=None, value_type=None,
                 **kw):
        super(I18NField, self).__init__(
            key_type=TextLine(),
            value_type=value_type,
            **kw
        )
        self._default_language = default_language

    def _validate(self, value):
        if isinstance(value, storage.I18NDict):
            value = value.to_dict()
        super(I18NField, self)._validate(value)
        if self.required:
            if not value:
                raise RequiredMissing
            for lang in value.values():
                if lang:
                    return
            raise RequiredMissing


@implementer(interfaces.II18NTextLineField)
class I18NTextLine(I18NField):
    """Schema field used to define an I18n textline property"""

    def __init__(self, key_type=None, value_type=None, value_constraint=None,
                 value_min_length=0, value_max_length=None, **kw):
        super(I18NTextLine, self).__init__(
            value_type=TextLine(
                constraint=value_constraint,
                min_length=value_min_length,
                max_length=value_max_length,
                required=False
            ),
            **kw
        )


@implementer(interfaces.II18NTextField)
class I18NText(I18NField):
    """Schema field used to define an I18n text property"""

    def __init__(self, key_type=None, value_type=None, value_constraint=None,
                 value_min_length=0, value_max_length=None, **kw):
        super(I18NText, self).__init__(
            value_type=Text(
                constraint=value_constraint,
                min_length=value_min_length,
                max_length=value_max_length,
                required=False
            ),
            **kw
        )



# -*- coding: utf-8 -*-
"""Widget interface definitions."""

# zope imports
from z3c.form.interfaces import IWidget


class II18NWidget(IWidget):
    """I18N base widget."""

    def sorted_languages():
        """Returns a list of the language codes in a sorted order."""

    def languages():
        """Returns a dictionary of language codes with language names."""

    def current():
        """Return the current display language."""

    def default_value():
        """Return the default value if it exists for the current widget."""

    def initWidget(widget, language):
        """Initialize the subwidget for the given language."""

    def updateWidget(widget, language):
        """Update the subwidget for the given language."""

    def getWidget(language):
        """Get the subwidget for the given language."""

    def getValue(language):
        """Get the value for the given language."""

    def hasValue(language):
        """Return whether a value exists for the given language."""


class II18NTextWidget(II18NWidget):
    """I18N text widget."""


class II18NTextAreaWidget(II18NWidget):
    """I18N textarea widget."""



# -*- coding: utf-8 -*-
"""Data converter."""

# zope imports
from z3c.form.converter import BaseDataConverter
from z3c.form.interfaces import IWidget
from zope.component import adapts

# local imports
from ps.zope.i18nfield.interfaces import II18NField


class I18NFieldDataConverter(BaseDataConverter):
    """Base data converter for I18N fields."""

    adapts(II18NField, IWidget)

    def toWidgetValue(self, value):
        if value is None:
            return {}

        # Return the dict with empty values filtered out for the special
        # cases when being used within the plone.registry.
        return dict([(k, v) for k, v in value.items() if v])

    def toFieldValue(self, value):
        if isinstance(value, basestring):
            return {}
        return dict([(k, v) for k, v in value.items() if v])



# -*- coding: utf-8 -*-
"""I18N index."""

# zope imports
from ps.zope.i18nfield import utils
from ps.zope.i18nfield.storage import I18NDict
from z3c.indexer.index import FieldIndex


class I18NFieldIndex(FieldIndex):

    def clear(self):
        """Initialize forward and reverse mappings."""
        super(I18NFieldIndex, self).clear()
        self._indices = self.family.OO.BTree()

    def documentCount(self):
        """See interface IStatistics"""
        index = self._indices.get(utils.get_language())
        if index:
            return index.documentCount()
        return super(I18NFieldIndex, self).documentCount()

    def wordCount(self):
        """See interface IStatistics"""
        index = self._indices.get(utils.get_language())
        if index:
            return index.wordCount()
        return super(I18NFieldIndex, self).wordCount()

    def sort(self, docids, reverse=False, limit=None):
        index = self._indices.get(utils.get_language())
        if index:
            return index.sort(docids, reverse, limit)
        return super(I18NFieldIndex, self).sort(docids, reverse, limit)

    def applyEq(self, value):
        index = self._indices.get(utils.get_language())
        if index:
            return index.applyEq(value)
        return super(I18NFieldIndex, self).applyEq(value)

    def applyNotEq(self, not_value):
        index = self._indices.get(utils.get_language())
        if index:
            return index.applyNotEq(not_value)
        return super(I18NFieldIndex, self).applyNotEq(not_value)

    def applyBetween(self, min_value, max_value, exclude_min=False,
                     exclude_max=False):
        index = self._indices.get(utils.get_language())
        if index:
            return index.applyBetween(
                min_value,
                max_value,
                exclude_min,
                exclude_max,
            )
        return super(I18NFieldIndex, self).applyBetween(
            min_value,
            max_value,
            exclude_min,
            exclude_max,
        )

    def applyGe(self, min_value, exclude_min=False):
        index = self._indices.get(utils.get_language())
        if index:
            return index.applyGe(min_value, exclude_min)
        return super(I18NFieldIndex, self).applyGe(min_value, exclude_min)

    def applyLe(self, max_value, exclude_max=False):
        index = self._indices.get(utils.get_language())
        if index:
            return index.applyLe(max_value, exclude_max)
        return super(I18NFieldIndex, self).applyLe(max_value, exclude_max)

    def applyIn(self, values):
        index = self._indices.get(utils.get_language())
        if index:
            return index.applyIn(values)
        return super(I18NFieldIndex, self).applyIn(values)

    def doIndex(self, oid, value):
        """Index a value by its object id."""
        if isinstance(value, I18NDict):
            for lang in utils.available_languages():
                lang_val = value.get_for_language(lang)
                if lang_val is None:
                    continue
                index = self._indices.get(lang)
                if index is None:
                    index = self._indices[lang] = FieldIndex()
                index.doIndex(oid, lang_val)
            return

    def doUnIndex(self, oid):
        """Unindex a value by its object id."""
        for lang in utils.available_languages():
            index = self._indices.get(lang)
            if index:
                index.doUnIndex(oid)



# -*- coding: utf-8 -*-
"""Schema interfaces."""

# zope imports
from zope.i18n.interfaces import ILanguageAvailability as \
    IBaseLanguageAvailability
from zope.interface import Interface
from zope.schema import interfaces


class ILanguageAvailability(IBaseLanguageAvailability):
    """A list of available languages."""

    def getDefaultLanguage(combined=False):
        """Return the system default language."""

    def getLanguages(combined=False):
        """Return a sequence of Language objects for available languages."""

    def getLanguageListing(combined=False):
        """Return a sequence of language code and language name tuples."""


class II18NField(interfaces.IDict):
    """Marker interface used to identify I18N properties."""


class II18NTextLineField(II18NField):
    """Marker interface used to identify I18N textline properties."""


class II18NTextField(II18NField):
    """Marker interface used to identify I18N text properties."""


class II18NDictReader(Interface):
    """Default value dict reading methods"""

    def __unicode__():
        """Create a unicode representation of the dictionary."""

    def __str__():
        """Create a string representation of the dictionary."""

    def __nonzero__():
        """Return whether the dictionary is considered empty or not."""

    def keys():
        """Get dict keys"""

    def values():
        """Get dict values"""

    def items():
        """Get dict items"""

    def get_for_language(language):
        """Get the value for a specific language using the appropriate default
        if it is a required field.
        """

    def copy():
        """Return an exact copy of the given dict with all associated
        properties."""

    def to_dict():
        """Return a plain dict representation of the object."""

    def to_text():
        """Return a string of all text values appended together."""


class II18NDictWriter(Interface):
    """Default value dict writing methods"""

    def __delitem__(key):
        """Delete specified key from dict"""

    def __setitem__(key, value):
        """Set specified key with specified value"""

    def clear():
        """Remove all dict values"""

    def add(language, value):
        """Add a value for the given language."""

    def remove(language):
        """Remove the value for the given language."""

    def update(*args, **kwargs):
        """Update dict with values from specified dict"""

    def setdefault(key, failobj=None):
        """Get given key from dict or set it with specified value"""

    def pop(key, *args):
        """Remove given key from dict and return its value"""

    def popitem():
        """Pop last item from dict"""


class II18NDict(II18NDictReader, II18NDictWriter):
    """Default value dict marker interface"""



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages
import os

version = '0.1.3.1'


def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()

long_description = (
    read('README.rst')
    + '\n' +
    read('CHANGES.rst'))

setup(
    name='ps.zope.i18nfield',
    version=version,
    description="A zope.schema field for inline translations.",
    long_description=long_description,
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Web Environment",
        "Framework :: Zope3",
        "Intended Audience :: Other Audience",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Zope",
    ],
    keywords='zope zope3 schema i18n z3c.form',
    author='Propertyshelf, Inc.',
    author_email='development@propertyshelf.com',
    url='https://github.com/propertyshelf/ps.zope.i18nfield',
    download_url='http://pypi.python.org/pypi/ps.zope.i18nfield',
    license='GPL',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    namespace_packages=['ps', 'ps.zope'],
    zip_safe=False,
    include_package_data=True,
    extras_require=dict(
        test=[
            'unittest2',
            'z3c.form [test]',
            'zc.buildout',
            'zope.browserpage',
            'zope.publisher',
            'zope.testing',
            'zope.traversing',
        ],
    ),
    install_requires=[
        'setuptools',
        'ZODB3',
        'z3c.indexer',
        'zope.globalrequest',
        'zope.i18n',
        'zope.interface',
        'zope.schema',
    ],
    entry_points="""
    # -*- Entry points: -*-
    """,
)



# -*- coding: utf-8 -*-
"""Helper methods for the I18N field."""

# python imports
import codecs

# zope imports
from zope.component import queryUtility
from zope.globalrequest import getRequest
from zope.i18n.negotiator import negotiator

# local imports
from ps.zope.i18nfield.interfaces import ILanguageAvailability

DEFAULT_LANGUAGE = u'en'


def to_unicode(value, default=u''):
    """Get specified value converted to unicode, or an empty unicode string if
    value is empty

    :param value: [required] text to be checked
    :type value: str or unicode
    :param default: default value
    :return: value, or default if value is empty
    :rtype: unicode
    """
    try:
        if isinstance(value, unicode):
            return value
        return codecs.decode(value or default)
    except:
        return codecs.decode(value or default, 'latin1')


def get_language(request=None):
    if request is None:
        request = getRequest()
    try:
        return negotiator.getLanguage(available_languages(), request)
    except TypeError:
        return get_default_language()


def get_default_language():
    """Return the default fallback languages."""
    utility = queryUtility(ILanguageAvailability)
    if utility is not None:
        return utility.getDefaultLanguage()
    return DEFAULT_LANGUAGE


def available_languages():
    """Return a list of available languages."""
    utility = queryUtility(ILanguageAvailability)
    if utility is not None:
        return utility.getAvailableLanguages()
    return [DEFAULT_LANGUAGE]
